@@ -31,20 +31,24 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/intstr"
 
+	network "knative.dev/networking/pkg"
 	"knative.dev/networking/pkg/apis/networking"
 	"knative.dev/pkg/kmeta"
 	"knative.dev/pkg/logging"
 	"knative.dev/pkg/metrics"
-	_ "knative.dev/pkg/metrics/testing"
 	"knative.dev/pkg/ptr"
 	"knative.dev/pkg/system"
-	_ "knative.dev/pkg/system/testing"
 	tracingconfig "knative.dev/pkg/tracing/config"
+	apicfg "knative.dev/serving/pkg/apis/config"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
-	asconfig "knative.dev/serving/pkg/autoscaler/config"
+	"knative.dev/serving/pkg/autoscaler/config/autoscalerconfig"
 	"knative.dev/serving/pkg/deployment"
-	"knative.dev/serving/pkg/network"
+	"knative.dev/serving/pkg/queue"
+	"knative.dev/serving/pkg/reconciler/revision/config"
+
+	_ "knative.dev/pkg/metrics/testing"
+	_ "knative.dev/pkg/system/testing"
 )
 
 var (
@@ -63,14 +67,26 @@ var (
 	}}
 
 	// The default CM values.
-	logConfig        logging.Config
-	traceConfig      tracingconfig.Config
-	obsConfig        metrics.ObservabilityConfig
-	deploymentConfig deployment.Config
-	asConfig         = asconfig.Config{
+	asConfig = autoscalerconfig.Config{
 		InitialScale:          1,
 		AllowZeroInitialScale: false,
 	}
+	deploymentConfig deployment.Config
+	logConfig        logging.Config
+	obsConfig        metrics.ObservabilityConfig
+	traceConfig      tracingconfig.Config
+	defaults, _      = apicfg.NewDefaultsConfigFromMap(nil)
+	revCfg           = config.Config{
+		Config: &apicfg.Config{
+			Autoscaler: &asConfig,
+			Defaults:   defaults,
+		},
+		Deployment:    &deploymentConfig,
+		Logging:       &logConfig,
+		Network:       &network.Config{},
+		Observability: &obsConfig,
+		Tracing:       &traceConfig,
+	}
 )
 
 const testProbeJSONTemplate = `{"tcpSocket":{"port":%d,"host":"127.0.0.1"}}`
@@ -80,11 +96,12 @@ func TestMakeQueueContainer(t *testing.T) {
 		name string
 		rev  *v1.Revision
 		lc   logging.Config
+		nc   network.Config
 		oc   metrics.ObservabilityConfig
 		dc   deployment.Config
 		want corev1.Container
 	}{{
-		name: "no owner no autoscaler single",
+		name: "autoscaler single",
 		rev: revision("bar", "foo",
 			withContainers(containers),
 			withContainerConcurrency(1)),
@@ -103,8 +120,7 @@ func TestMakeQueueContainer(t *testing.T) {
 					ContainerPort: 1955,
 					Name:          string(networking.ProtocolH2C),
 				}},
-			}}),
-			withContainerConcurrency(1)),
+			}})),
 		dc: deployment.Config{
 			QueueSidecarImage: "alpine",
 		},
@@ -112,18 +128,16 @@ func TestMakeQueueContainer(t *testing.T) {
 			c.Image = "alpine"
 			c.Ports = append(queueNonServingPorts, queueHTTP2Port)
 			c.Env = env(map[string]string{
-				"USER_PORT":             "1955",
-				"QUEUE_SERVING_PORT":    "8013",
-				"CONTAINER_CONCURRENCY": "1",
+				"USER_PORT":          "1955",
+				"QUEUE_SERVING_PORT": "8013",
 			})
 		}),
 	}, {
 		name: "service name in labels",
 		rev: revision("bar", "foo",
 			withContainers(containers),
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.ObjectMeta.Labels = map[string]string{
+				revision.Labels = map[string]string{
 					serving.ServiceLabelKey: "svc",
 				}
 			}),
@@ -157,8 +171,7 @@ func TestMakeQueueContainer(t *testing.T) {
 	}, {
 		name: "logging configuration as env var",
 		rev: revision("this", "log",
-			withContainers(containers),
-			withContainerConcurrency(1)),
+			withContainers(containers)),
 		lc: logging.Config{
 			LoggingConfig: "The logging configuration goes here",
 			LoggingLevel: map[string]zapcore.Level{
@@ -186,8 +199,7 @@ func TestMakeQueueContainer(t *testing.T) {
 	}, {
 		name: "request log configuration as env var",
 		rev: revision("bar", "foo",
-			withContainers(containers),
-			withContainerConcurrency(1)),
+			withContainers(containers)),
 		oc: metrics.ObservabilityConfig{
 			RequestLogTemplate:    "test template",
 			EnableProbeRequestLog: true,
@@ -198,11 +210,26 @@ func TestMakeQueueContainer(t *testing.T) {
 				"SERVING_ENABLE_PROBE_REQUEST_LOG": "true",
 			})
 		}),
+	}, {
+		name: "disabled request log configuration as env var",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		oc: metrics.ObservabilityConfig{
+			RequestLogTemplate:    "test template",
+			EnableProbeRequestLog: false,
+			EnableRequestLog:      false,
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"SERVING_REQUEST_LOG_TEMPLATE":     "test template",
+				"SERVING_ENABLE_REQUEST_LOG":       "false",
+				"SERVING_ENABLE_PROBE_REQUEST_LOG": "false",
+			})
+		}),
 	}, {
 		name: "request metrics backend as env var",
 		rev: revision("bar", "foo",
-			withContainers(containers),
-			withContainerConcurrency(1)),
+			withContainers(containers)),
 		oc: metrics.ObservabilityConfig{
 			RequestMetricsBackend: "prometheus",
 		},
@@ -214,8 +241,7 @@ func TestMakeQueueContainer(t *testing.T) {
 	}, {
 		name: "enable profiling",
 		rev: revision("bar", "foo",
-			withContainers(containers),
-			withContainerConcurrency(1)),
+			withContainers(containers)),
 		oc: metrics.ObservabilityConfig{EnableProfiling: true},
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
@@ -227,17 +253,19 @@ func TestMakeQueueContainer(t *testing.T) {
 		name: "custom TimeoutSeconds",
 		rev: revision("bar", "foo",
 			withContainers(containers),
-			withContainerConcurrency(1)),
+			func(revision *v1.Revision) {
+				revision.Spec.TimeoutSeconds = ptr.Int64(99)
+			},
+		),
 		want: queueContainer(func(c *corev1.Container) {
 			c.Env = env(map[string]string{
-				"REVISION_TIMEOUT_SECONDS": "45",
+				"REVISION_TIMEOUT_SECONDS": "99",
 			})
 		}),
 	}, {
 		name: "default resource config",
 		rev: revision("bar", "foo",
-			withContainers(containers),
-			withContainerConcurrency(1)),
+			withContainers(containers)),
 		dc: deployment.Config{
 			QueueSidecarCPURequest: &deployment.QueueSidecarCPURequestDefault,
 		},
@@ -251,8 +279,7 @@ func TestMakeQueueContainer(t *testing.T) {
 	}, {
 		name: "overridden resources",
 		rev: revision("bar", "foo",
-			withContainers(containers),
-			withContainerConcurrency(1)),
+			withContainers(containers)),
 		dc: deployment.Config{
 			QueueSidecarCPURequest:              resourcePtr(resource.MustParse("123m")),
 			QueueSidecarEphemeralStorageRequest: resourcePtr(resource.MustParse("456M")),
@@ -268,6 +295,20 @@ func TestMakeQueueContainer(t *testing.T) {
 				corev1.ResourceMemory: resource.MustParse("789m"),
 			}
 		}),
+	}, {
+		name: "collector address as env var",
+		rev: revision("bar", "foo",
+			withContainers(containers)),
+		oc: metrics.ObservabilityConfig{
+			RequestMetricsBackend:   "opencensus",
+			MetricsCollectorAddress: "otel:55678",
+		},
+		want: queueContainer(func(c *corev1.Container) {
+			c.Env = env(map[string]string{
+				"SERVING_REQUEST_METRICS_BACKEND": "opencensus",
+				"METRICS_COLLECTOR_ADDRESS":       "otel:55678",
+			})
+		}),
 	}}
 
 	for _, test := range tests {
@@ -280,7 +321,13 @@ func TestMakeQueueContainer(t *testing.T) {
 					}},
 				}
 			}
-			got, err := makeQueueContainer(test.rev, &test.lc, &traceConfig, &test.oc, &test.dc)
+			cfg := &config.Config{
+				Tracing:       &traceConfig,
+				Logging:       &test.lc,
+				Observability: &test.oc,
+				Deployment:    &test.dc,
+			}
+			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error:", err)
 			}
@@ -291,8 +338,8 @@ func TestMakeQueueContainer(t *testing.T) {
 			})
 			sortEnv(got.Env)
 			sortEnv(test.want.Env)
-			if diff := cmp.Diff(test.want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-				t.Error("makeQueueContainer (-want, +got) =", diff)
+			if got, want := *got, test.want; !cmp.Equal(got, want, quantityComparer) {
+				t.Errorf("makeQueueContainer (-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 			}
 		})
 	}
@@ -307,9 +354,8 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 	}{{
 		name: "resources percentage in annotations",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "20",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
@@ -333,9 +379,8 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 	}, {
 		name: "resources percentage in annotations smaller than min allowed",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "0.2",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
@@ -359,9 +404,8 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 	}, {
 		name: "invalid resources percentage in annotations uses defaults",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "foo",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
@@ -387,9 +431,8 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 	}, {
 		name: "resources percentage in annotations bigger than than math.MaxInt64",
 		rev: revision("bar", "foo",
-			withContainerConcurrency(1),
 			func(revision *v1.Revision) {
-				revision.ObjectMeta.Annotations = map[string]string{
+				revision.Annotations = map[string]string{
 					serving.QueueSideCarResourcePercentageAnnotation: "100",
 				}
 				revision.Spec.PodSpec.Containers = []corev1.Container{{
@@ -416,7 +459,9 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			got, err := makeQueueContainer(test.rev, &logConfig, &traceConfig, &obsConfig, &test.dc)
+			cfg := (&revCfg).DeepCopy()
+			cfg.Deployment = &test.dc
+			got, err := makeQueueContainer(test.rev, cfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error:", err)
 			}
@@ -426,8 +471,8 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 			})
 			sortEnv(got.Env)
 			sortEnv(test.want.Env)
-			if diff := cmp.Diff(test.want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-				t.Error("makeQueueContainer (-want, +got) =", diff)
+			if got, want := *got, test.want; !cmp.Equal(got, want, quantityComparer) {
+				t.Errorf("makeQueueContainer (-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 			}
 		})
 	}
@@ -435,7 +480,6 @@ func TestMakeQueueContainerWithPercentageAnnotation(t *testing.T) {
 
 func TestProbeGenerationHTTPDefaults(t *testing.T) {
 	rev := revision("bar", "foo",
-		withContainerConcurrency(1),
 		func(revision *v1.Revision) {
 			revision.Spec.PodSpec.Containers = []corev1.Container{{
 				Name: servingContainerName,
@@ -460,7 +504,7 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 				Scheme: corev1.URISchemeHTTP,
 				HTTPHeaders: []corev1.HTTPHeader{{
 					Name:  network.KubeletProbeHeaderName,
-					Value: "queue",
+					Value: queue.Name,
 				}},
 			},
 		},
@@ -488,13 +532,13 @@ func TestProbeGenerationHTTPDefaults(t *testing.T) {
 		}
 	})
 
-	got, err := makeQueueContainer(rev, &logConfig, &traceConfig, &obsConfig, &deploymentConfig)
+	got, err := makeQueueContainer(rev, &revCfg)
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
 	sortEnv(got.Env)
-	if diff := cmp.Diff(want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-		t.Error("makeQueueContainer(-want, +got) =", diff)
+	if got, want := *got, want; !cmp.Equal(got, want, quantityComparer) {
+		t.Errorf("makeQueueContainer(-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 	}
 }
 
@@ -503,7 +547,6 @@ func TestProbeGenerationHTTP(t *testing.T) {
 	const probePath = "/health"
 
 	rev := revision("bar", "foo",
-		withContainerConcurrency(1),
 		func(revision *v1.Revision) {
 			revision.Spec.PodSpec.Containers = []corev1.Container{{
 				Name: servingContainerName,
@@ -532,7 +575,7 @@ func TestProbeGenerationHTTP(t *testing.T) {
 				Scheme: corev1.URISchemeHTTPS,
 				HTTPHeaders: []corev1.HTTPHeader{{
 					Name:  network.KubeletProbeHeaderName,
-					Value: "queue",
+					Value: queue.Name,
 				}},
 			},
 		},
@@ -560,13 +603,13 @@ func TestProbeGenerationHTTP(t *testing.T) {
 		}
 	})
 
-	got, err := makeQueueContainer(rev, &logConfig, &traceConfig, &obsConfig, &deploymentConfig)
+	got, err := makeQueueContainer(rev, &revCfg)
 	if err != nil {
 		t.Fatal("makeQueueContainer returned error")
 	}
 	sortEnv(got.Env)
-	if diff := cmp.Diff(want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-		t.Error("makeQueueContainer(-want, +got) =", diff)
+	if got, want := *got, want; !cmp.Equal(got, want, quantityComparer) {
+		t.Errorf("makeQueueContainer(-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 	}
 }
 
@@ -590,8 +633,7 @@ func TestTCPProbeGeneration(t *testing.T) {
 			SuccessThreshold: 3,
 		},
 		rev: v1.RevisionSpec{
-			ContainerConcurrency: ptr.Int64(1),
-			TimeoutSeconds:       ptr.Int64(45),
+			TimeoutSeconds: ptr.Int64(45),
 			PodSpec: corev1.PodSpec{
 				Containers: []corev1.Container{{
 					Name: servingContainerName,
@@ -623,8 +665,7 @@ func TestTCPProbeGeneration(t *testing.T) {
 	}, {
 		name: "tcp defaults",
 		rev: v1.RevisionSpec{
-			ContainerConcurrency: ptr.Int64(1),
-			TimeoutSeconds:       ptr.Int64(45),
+			TimeoutSeconds: ptr.Int64(45),
 			PodSpec: corev1.PodSpec{
 				Containers: []corev1.Container{{
 					Name: servingContainerName,
@@ -675,8 +716,7 @@ func TestTCPProbeGeneration(t *testing.T) {
 			InitialDelaySeconds: 3,
 		},
 		rev: v1.RevisionSpec{
-			ContainerConcurrency: ptr.Int64(1),
-			TimeoutSeconds:       ptr.Int64(45),
+			TimeoutSeconds: ptr.Int64(45),
 			PodSpec: corev1.PodSpec{
 				Containers: []corev1.Container{{
 					Name: servingContainerName,
@@ -727,41 +767,43 @@ func TestTCPProbeGeneration(t *testing.T) {
 				Value: string(wantProbeJSON),
 			})
 
-			got, err := makeQueueContainer(testRev, &logConfig, &traceConfig, &obsConfig, &deploymentConfig)
+			got, err := makeQueueContainer(testRev, &revCfg)
 			if err != nil {
 				t.Fatal("makeQueueContainer returned error")
 			}
 			sortEnv(got.Env)
 			sortEnv(test.want.Env)
-			if diff := cmp.Diff(test.want, *got, cmp.AllowUnexported(resource.Quantity{})); diff != "" {
-				t.Error("makeQueueContainer (-want, +got) =", diff)
+			if got, want := *got, test.want; !cmp.Equal(want, got, quantityComparer) {
+				t.Errorf("makeQueueContainer (-want, +got) =\n%s", cmp.Diff(want, got, quantityComparer))
 			}
 		})
 	}
 }
 
 var defaultEnv = map[string]string{
-	"SERVING_NAMESPACE":                     "foo",
-	"SERVING_SERVICE":                       "",
-	"SERVING_CONFIGURATION":                 "",
-	"SERVING_REVISION":                      "bar",
-	"CONTAINER_CONCURRENCY":                 "1",
+	"CONTAINER_CONCURRENCY":                 "0",
+	"ENABLE_PROFILING":                      "false",
+	"METRICS_DOMAIN":                        metrics.Domain(),
+	"METRICS_COLLECTOR_ADDRESS":             "",
+	"QUEUE_SERVING_PORT":                    "8012",
 	"REVISION_TIMEOUT_SECONDS":              "45",
+	"SERVING_CONFIGURATION":                 "",
+	"SERVING_ENABLE_PROBE_REQUEST_LOG":      "false",
+	"SERVING_ENABLE_REQUEST_LOG":            "false",
 	"SERVING_LOGGING_CONFIG":                "",
 	"SERVING_LOGGING_LEVEL":                 "",
-	"TRACING_CONFIG_BACKEND":                "",
-	"TRACING_CONFIG_ZIPKIN_ENDPOINT":        "",
-	"TRACING_CONFIG_STACKDRIVER_PROJECT_ID": "",
-	"TRACING_CONFIG_SAMPLE_RATE":            "0",
-	"TRACING_CONFIG_DEBUG":                  "false",
+	"SERVING_NAMESPACE":                     "foo",
 	"SERVING_REQUEST_LOG_TEMPLATE":          "",
 	"SERVING_REQUEST_METRICS_BACKEND":       "",
-	"USER_PORT":                             strconv.Itoa(v1.DefaultUserPort),
+	"SERVING_REVISION":                      "bar",
+	"SERVING_SERVICE":                       "",
 	"SYSTEM_NAMESPACE":                      system.Namespace(),
-	"METRICS_DOMAIN":                        metrics.Domain(),
-	"QUEUE_SERVING_PORT":                    "8012",
-	"ENABLE_PROFILING":                      "false",
-	"SERVING_ENABLE_PROBE_REQUEST_LOG":      "false",
+	"TRACING_CONFIG_BACKEND":                "",
+	"TRACING_CONFIG_DEBUG":                  "false",
+	"TRACING_CONFIG_SAMPLE_RATE":            "0",
+	"TRACING_CONFIG_STACKDRIVER_PROJECT_ID": "",
+	"TRACING_CONFIG_ZIPKIN_ENDPOINT":        "",
+	"USER_PORT":                             strconv.Itoa(v1.DefaultUserPort),
 }
 
 func probeJSON(container *corev1.Container) string {
